#####################################################################################################################
# Copyright IBM
#
# This PERL script was created by Stuart Poulin of Rational Software 2008.
#
#######################################################################################################################
# Wrapper class to ClearQuest BuildEntity
# The Idea is to simplify creating and updating records for ALM 
# IBM Corporation September 29th, 2008
# Stuart Poulin
# 
# my ($act) = $bl->get("Activities");
#
our $VERSION = "2008.1002";
package create_record;
use strict;
use Carp;
our $AUTOLOAD;

my $allowed = {
 AddAttachmentFieldValue => 1,
 AddFieldValue => 1,
 BeginNewFieldUpdateGroup => 1,
# Commit => 1,
 DeleteAttachmentFieldValue => 1,
 DeleteFieldValue => 1,
 EditAttachmentFieldDescription => 1,
 EditEntity => 1,
 FireNamedHook => 1,
 GetActionName => 1,
 GetActionType => 1,
 GetAllDuplicates => 1,
 GetAllFieldValues => 1,
 GetAttachmentDisplayNameHeader => 1,
 GetDbId => 1,
 GetDefaultActionName => 1,
 GetDisplayName => 1,
 GetDuplicates => 1,
 GetEntityDefName => 1,
 GetFieldChoiceList => 1,
 GetFieldChoiceType => 1,
 GetFieldMaxLength => 1,
 GetFieldNames => 1,
 GetFieldOriginalValue => 1,
 GetFieldRequiredness => 1,
 GetFieldStringValue => 1,
 GetFieldStringValueAsList => 1,
 GetFieldStringValues => 1,
 GetFieldsUpdatedThisAction => 1,
 GetFieldsUpdatedThisEntireAction => 1,
 GetFieldsUpdatedThisGroup => 1,
 GetFieldsUpdatedThisSetValue => 1,
 GetFieldType => 1,
 GetFieldValue => 1,
 GetInvalidFieldValues => 1,
 GetLegalAccessibleActionDefNames => 1,
 GetLegalActionDefNames => 1,
 GetOriginal => 1,
 GetOriginalID => 1,
 GetSession => 1,
 GetType => 1,
 HasDuplicates => 1,
 InvalidateFieldChoiceList => 1,
 IsDuplicate => 1,
 IsEditable => 1,
 IsOriginal => 1,
 LoadAttachment => 1,
 LookupStateName => 1,
 Reload => 1,
 Revert => 1,
 SetFieldChoiceList => 1,
 SetFieldRequirednessForCurrentAction => 1,
 SetFieldValue => 1,
 SetFieldValues => 1,
 SiteHasMastership => 1,
# Validate => 1,
};
 
sub new { 
	my $class = shift;
	my $self = { setFieldsAndValidate => 0, setFieldsAndCommit => 0,@_};
	bless($self, $class);
	if ($self->{id} ) { # Edit mode
		$self->{entity} = $self->{session}->GetEntity($self->{type}, $self->{id});

	}
	else { # Create new mode
		$self->{entity}=$self->{session}->BuildEntity($self->{type});
	}
	if (exists $self->{fields} ) {
		$self->set_fields($self->{fields});
		$self->validate() unless $self->{SetFieldsAndValidate} == 0;
		$self->commit() unless $self->{SetFieldsAndCommit} == 0;
	}
	return $self;
	return undef;
}

sub set_fields {
	my $self = shift;
	my $fields = shift; # not a hash, needs to be ordered pairs for hook code order compliance.
	for (my $i = 0; $i < @$fields; $i+=2) {
		my $field = $fields->[$i];
		my $val = $fields->[$i+1];
		if (ref $val eq "ARRAY" ) {
			for my $a_val (@$val) {
				my $results = $self->{entity}->AddFieldValue($field,$a_val);
				carp "Results=$results" if $results;
			}
		}
		else {
			my $results = $self->{entity}->SetFieldValue($field,$val);
			carp "Results=$results" if $results;
		}

	}

}	

sub Validate {
	my $self = shift;
	my $entity = $self->{entity};
	my $status;
	eval {$status = $entity->Validate()};
	if (($@) || ($status)) {
		carp "New record type $self->{type} failed to Validate: @_ : $status";
		$entity->Revert();
		return undef;
	}
	1;
}

sub Commit {
	my $self = shift;
	my $entity = $self->{entity};
	my $status;
	eval {$status = $entity->Commit()};
	if (($@) || ($status)) {
		carp "New record type $self->{type} failed to Commit: @_ : $status";
		$entity->Revert();
		return undef;
	}
	1;
}

sub AUTOLOAD {
        my $self = shift;
        my $type = ref($self)
                    or croak "$self is not an object";

        my $name = $AUTOLOAD;
        $name =~ s/.*://;   # strip fully-qualified portion

	unless (exists $allowed->{$name} ) {
	    croak "'$name' is not an allowed operation for this object";
	}

	#*$name = sub { my $self=shift; $self->{mox}->$name(@_) };
	my $str = "sub $name { my \$self=shift; \$self->{entity}->$name(\@_) }";
	carp "$str\n" if $self->{debug};
	eval $str;
	unshift @_,$self;
	goto &$name;
}

1;
